#include "rclcpp/rclcpp.hpp"

#include <chrono>
#include <string>
#include <memory>
#include <thread>

using namespace std::chrono_literals;

class TimerNode : public rclcpp::Node
{
public:
  TimerNode(std::string timer_node)
  : Node(timer_node)
  {
    callback_group_ = create_callback_group(rclcpp::CallbackGroupType::MutuallyExclusive, false);

    timer1_ = this->create_wall_timer(
      0s,
      [this]() -> void {
        timer1_->cancel();
        RCLCPP_INFO(get_logger(), "Hey from outer wall timer");

        timer2_ = this->create_wall_timer(
          0s,
          [this]() -> void {
            timer2_->cancel();
            RCLCPP_INFO(get_logger(), "Hey from nested wall timer");
          },
          callback_group_);

        RCLCPP_INFO(get_logger(), "Inner timer created");
      });
      RCLCPP_INFO(get_logger(), "Outer timer created");

      executor_ = std::make_shared<rclcpp::executors::SingleThreadedExecutor>();
      executor_->add_callback_group(callback_group_, get_node_base_interface());
      service_thread_ = std::make_unique<std::thread>([&]() {executor_->spin();});

      std::this_thread::sleep_for(std::chrono::milliseconds(1000));
  }

  ~TimerNode()
  {
    executor_->cancel();
    service_thread_->join();
  }

private:
  rclcpp::TimerBase::SharedPtr timer1_, timer2_;

  rclcpp::CallbackGroup::SharedPtr callback_group_;
  rclcpp::Executor::SharedPtr executor_;
  std::unique_ptr<std::thread> service_thread_;
};

int main(int argc, char * argv[])
{
  const std::string & node_name("timer_node");
  auto logger = rclcpp::get_logger(node_name);

  rclcpp::init(argc, argv);
  auto node = std::make_shared<TimerNode>(node_name);
  rclcpp::spin(node);
  rclcpp::shutdown();

  return 0;
}
